class APIKeyMissingError(Exception):
    def __init__(self) -> None: ...

class ResponseFormatTypeMissingError(Exception):
    def __init__(self) -> None: ...

class InvalidInputError(Exception):
    def __init__(self, message: str) -> None: ...
